#!/bin/bash
#

# buildTBB.sh

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e7|e9|i1> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=tbb
origpkgver=v4_4_3
pkgver=${origpkgver}
ssibuildshims_version=v0_09_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=tbb44_20150728oss

get_this_dir

get_ssibuildshims

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [[ "${extraqual}" == "opt" ]] || [[ "${extraqual}" == "prof" ]]
then
    reltype="release"
elif [ "${extraqual}" = "debug" ]
then
    reltype="debug"
fi

mkdir -p ${pkgdir}/lib
if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`
extra_command="-j ${ncore}"

if [[ "${basequal}" == e[79] ]]; then
  compiler=gcc
  extracommand="cpp1y=1 ${extracommand}"
elif [[ "${basequal}" == "i1" ]]; then
  compiler=icc
fi

if [[ "${compiler}" == icc ]]
then
  # Clean up variables set by icc setup for tbb.
  for var in $(printenv | grep tbb | grep -v PWD | cut -d= -f1); do
    eval ${var}=`dropit -D -p $(eval echo \\${\${var}}) tbb`
    if [[ -z "$(eval echo \\$\${var})" ]]; then
      eval unset ${var}
    fi
  done
  extra_command="${extra_command} offload=mic"
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup root
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "ERROR: fake setup failed"

set -x

# TBB requires an in-source build
cd ${pkgdir}
tar xf ${tardir}/${pkgtarname}_src.tgz || die "ERROR: untar source failed."

cd ${pkgdir}/${pkgtarname}

patch --verbose -b -p0 < ${patchdir}/tbb.patch || die "ERROR: patch failed."

cd ${pkgdir}/${pkgtarname}/src

make compiler=${compiler} ${extra_command} tbb_${reltype} tbbmalloc_${reltype} || die "ERROR: make failed."

set +x

mv -v ../include "${pkgdir}"/

# Move things into the expected locations (must match UPS table file).
if [ ${OS1} = "Darwin" ]
then
  mv -v ${pkgdir}/${pkgtarname}/build/macos*/lib*.dylib ${pkgdir}/lib/
  ext=.dylib
else
  mv ${pkgdir}/${pkgtarname}/build/lin*/lib*.so* ${pkgdir}/lib/
  if [[ -d "${pkgdir}/${pkgtarname}/build/mic_icc_${reltype}" ]]; then
    mkdir ${pkgdir}/lib-mic
    mv -v "${pkgdir}/${pkgtarname}/build/mic_icc_${reltype}"/libtbb*.so* "${pkgdir}/lib-mic/"
  fi
  ext=.so
fi

mv -v "${pkgdir}/${pkgtarname}/doc" "${pkgdir}/"

# Convenience links for debug libraries.
if [ "${extraqual}" = "debug" ]; then
  suffix=_debug
fi

make compiler=${compiler} ${extra_command} clean

# 1. Replace linker stubs with soft links to avoid confusing CMake when
# trying to link to these libraries (not for OS X).
#
# 2. For debug, make soft links without _debug in the name for convenience.
for dir in "${pkgdir}/lib" "${pkgdir}/lib-mic"; do
  [[ -d "${dir}" ]] || continue
  pushd "${dir}" >/dev/null
  for lib in libtbb{,malloc{,_proxy}}"${suffix}${ext}"; do
    soname=$(grep -qI INPUT "${lib}" && cat "${lib}" | sed -e 's&^INPUT (\(.*\))$&\1&')
    [[ -n "${soname}" ]] && ln -svf "${soname}" "${lib}" # 1.
    [[ -n "${suffix}" ]] && ln -svf "${soname:-${lib}}" "${lib/${suffix}/}" # 2.
  done
  popd >/dev/null
done

# Source
cd "${pkgdir}/.."
[[ ! -d source ]] && mv "${pkgdir}/${pkgtarname}/src" source
rm -rf "${pkgdir}/${pkgtarname}"

# cmake configure files
cp -pr "${patchdir}/${package}" "${pkgdir}/lib/"
# Ensure version is set correctly.
config_files=("${pkgdir}/lib/tbb/cmake/tbb-config"{,-version}".cmake")
for config_file in "${config_files[@]}"; do
  perl -wapi\~ \
     -e 's&\%\%\%PKGVER\%\%\%&'"${pkgver}"'&;
         s&\%\%\%PKGDOTVER\%\%\%&'"${pkgdotver}"'&;' \
     "${config_file}"
  if grep -q -I -F -e "${pkgver}" -e "${pkgdotver}" "${config_file}"; then
    rm -f "${config_file}~"
  else
    die "Unable to update version in ${config_file}"
  fi
done

# ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "${package} is installed at ${TBB_FQ_DIR}"

# this must be last 
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
